﻿using System.Collections;                    // needed for ArrayList collection
using System.Windows.Forms;                  // needed so sound device knows its owner
using Microsoft.DirectX.DirectSound;         // needed for sound
using Microsoft.DirectX.AudioVideoPlayback;  // needed for music

namespace VDEngine.DirectX
{
    /// <summary>
    /// Sound and music of DirectSound and AudioVideoPlayback.
    /// </summary>
    class VDAudio
    {
        private Device _soundDevice;           // sound device
        private SecondaryBuffer _soundEffect;  // sound
        private ArrayList _soundEffectList;    // sound list
        private Audio _music;                  // music



        #region Initialization (Constructor)

        /// <summary>
        /// Initialize the game's audio.
        /// </summary>
        /// <param name="owner">Sound device's owner.</param>
        public VDAudio(Control owner)
        {
            _soundDevice = new Device();

            // decent control over sound
            _soundDevice.SetCooperativeLevel(owner, CooperativeLevel.Priority);

            // load up a sound effect
            BufferDescription bufferDescription = new BufferDescription();
            bufferDescription.ControlEffects = false;
            _soundEffect = new SecondaryBuffer("../../../soundeffect.wav", bufferDescription, _soundDevice);

            // keep a list of sound effects so they can be removed
            _soundEffectList = new ArrayList();

            // load up an mp3 music file
            _music = new Audio("../../../title.mp3");
            _music.Play();
        }

        #endregion



        #region Sound Management

        /// <summary>
        /// This is a temporary function that allows the testing of sound.
        /// </summary>
        public void TempOnClick()
        {
            // make multiple sounds at the same time (clone buffer)
            SecondaryBuffer _newSoundEffect = _soundEffect.Clone(_soundDevice);
            _newSoundEffect.Play(0, BufferPlayFlags.Default);
            _soundEffectList.Add(_newSoundEffect);
        }

        /// <summary>
        /// Manages all of the simultaneous sound effects.
        /// </summary>
        public void SoundEffectManagement()
        {
            if (_soundEffectList.Count > 0)
            {
                for (int n = 0; n < _soundEffectList.Count; n++)
                {
                    SecondaryBuffer _currentSoundEffect = (SecondaryBuffer) _soundEffectList[n];
                    if (!_currentSoundEffect.Status.Playing)
                    {
                        _currentSoundEffect.Dispose();
                        _soundEffectList.RemoveAt(n);
                    }
                }
            }

            // TODO: loop the music (this doesn't work... b/c the machine's too fast)
            /*if (_music.CurrentPosition == _lastMusicPosition)
            {
                _music.SeekCurrentPosition(0, SeekPositionFlags.AbsolutePositioning);
            }
            _lastMusicPosition = _music.CurrentPosition;*/
        }

        #endregion



        #region Shutdown

        /// <summary>
        /// Release the resources of the audio.
        /// </summary>
        public void Shutdown()
        {
            if (_soundDevice != null)
            {
                _soundDevice.Dispose();
                _soundDevice = null;
            }
            if (_music != null)
            {
                _music.Dispose();
                _music = null;
            }
        }

        #endregion
    }
}
