﻿using System;
using System.Collections;
using System.Drawing;
using Microsoft.DirectX;
using Direct3D = Microsoft.DirectX.Direct3D;
using VDEngine.Game;

// TODO:
// Particle System
// ---------------
//
// Position (X, Y, Z)
// Age (point size)
// Direction
// Speed
// Wind influence
// Gravity influence

namespace VDEngine.DirectX
{
    /// <summary>
    /// Main particle system
    /// </summary>
    public class VDParticleSystem
    {
        private ArrayList _particleList;
        private Particle[] _particles;

        struct VSprite
        {
            public Vector3 position;
            public Vector3 angle;
        }

        struct Particle
        {
            public float positionX;
            public float positionY;
            public float positionZ;
            public float pointSize;
            public int color;

            public Particle(float positionX, float positionY, float positionZ, float pointSize, int color)
            {
                this.positionX = positionX;
                this.positionY = positionY;
                this.positionZ = positionZ;
                this.pointSize = pointSize;
                this.color = color;
            }
        }

        /// <summary>
        /// Initializes the particle system.
        /// </summary>
        public VDParticleSystem()
        {
            _particleList = new ArrayList();
        }

        /// <summary>
        /// Accessor for the list of particles.
        /// </summary>
        public ArrayList PointList
        {
            get
            {
                return _particleList;
            }
        }

        /// <summary>
        /// Temporary firing particle system.
        /// </summary>
        /// <param name="player">Player information needed for the user's current position and direction.</param>
        public void FireBullets(VDPlayer player)
        {
            VSprite newSprite = new VSprite();
            newSprite.position = player.Location;
            newSprite.angle = player.Direction;
            _particleList.Add(newSprite);
        }

        /// <summary>
        /// Temporary position updater for our bullet particles.
        /// </summary>
        /// <param name="elapsedTime">Time that has gone by.</param>
        public void UpdatePosition(double elapsedTime)
        {
            _particles = new Particle[_particleList.Count];

            for (int n = 0; n < _particleList.Count; n++)
            {
                VSprite currentSprite = (VSprite) _particleList[n];
                currentSprite.position.X -= (float) Math.Sin(currentSprite.angle.X) * 1.0f * (float) elapsedTime;
                currentSprite.position.Z -= (float) Math.Cos(currentSprite.angle.X) * 1.0f * (float) elapsedTime;
                _particleList[n] = currentSprite;
                _particles[n] = new Particle(currentSprite.position.X,
                                             currentSprite.position.Y,
                                             currentSprite.position.Z,
                                             0.5f,
                                             Color.White.ToArgb());
            }
        }

        /// <summary>
        /// Renderer of the particle system.
        /// </summary>
        /// <param name="graphicsDevice">Direct3D graphics device.</param>
        public void Render(Direct3D.Device graphicsDevice)
        {
            // TODO: somehow set this up only once... but it still needs to know of the D3D device
            graphicsDevice.RenderState.PointSpriteEnable = true;       // point sprites enabled
            graphicsDevice.RenderState.PointScaleEnable = true;        // let Direct3D scale the sprite size
            // final dimension = ViewportHeight * PointSize * sqrt(1 / (A + B * DistanceFromCamera + C * DistanceFromCamera^2))
            //graphicsDevice.RenderState.PointSize = 0.0f
            graphicsDevice.RenderState.PointScaleA = 0.0f;   // A from the above equation
            graphicsDevice.RenderState.PointScaleB = 0.0f;   // B from the above equation
            graphicsDevice.RenderState.PointScaleC = 50.0f;  // C from the above equation

            if (_particleList.Count > 0)
            {
                graphicsDevice.DrawUserPrimitives(Direct3D.PrimitiveType.PointList, _particleList.Count, _particles);
            }
        }
    }
}
