﻿using System;
using System.Drawing;                         // needed for Color class
using Microsoft.DirectX;                      // needed for vector stuff
using Direct3D = Microsoft.DirectX.Direct3D;  // needed for Direct3D

namespace VDEngine.DirectX
{
    /// <summary>
    /// Maintains a collection of the textures.
    /// </summary>
    public class VDTexture
    {
        private Direct3D.CustomVertex.PositionTextured[] _vertices;  //
        private Direct3D.Texture _texture;                           //
        private Direct3D.Device _device;                             //
        private float _aspectRatio;

        /// <summary>
        /// Initializes the textures.
        /// </summary>
        /// <param name="device">Direct3D device.</param>
        /// <param name="aspectRatio">Window's aspect ratio.</param>
        public VDTexture(Direct3D.Device device, float aspectRatio)
        {
            _device = device;
            _aspectRatio = aspectRatio;
            TemporaryLoadTextures();
        }

        /// <summary>
        /// Accessor for the texture.
        /// </summary>
        public Direct3D.Texture Texture
        {
            get
            {
                return _texture;
            }
        }

        /// <summary>
        /// Accessor for the vertices of the texture.
        /// </summary>
        public Direct3D.CustomVertex.PositionTextured[] Vertices
        {
            get
            {
                return _vertices;
            }
        }

        /// <summary>
        /// TODO: Load texture from file or resources
        /// </summary>
        public void LoadTexture()
        {
        }

        /// <summary>
        /// TODO: Temporary for loading a test texture
        /// </summary>
        public void TemporaryLoadTextures()
        {
            _vertices = new Direct3D.CustomVertex.PositionTextured[6];

            /*_vertices[0].Position = new Vector3(13.333f, 10f, 0f);
            _vertices[0].Tu = 0;
            _vertices[0].Tv = 0;

            _vertices[1].Position = new Vector3(-13.333f, -10f, 0f);
            _vertices[1].Tu = 1;
            _vertices[1].Tv = 1;

            _vertices[2].Position = new Vector3(13.333f, -10f, 0f);
            _vertices[2].Tu = 0;
            _vertices[2].Tv = 1;

            _vertices[3].Position = new Vector3(-13.333f, -10.0f, 0f);
            _vertices[3].Tu = 1;
            _vertices[3].Tv = 1;

            _vertices[4].Position = new Vector3(13.333f, 10.0f, 0f);
            _vertices[4].Tu = 0;
            _vertices[4].Tv = 0;

            _vertices[5].Position = new Vector3(-13.333f, 10.0f, 0f);
            _vertices[5].Tu = 1;
            _vertices[5].Tv = 0;

            */
            _vertices[0].Position = new Vector3(5.0f * _aspectRatio, 5.0f, 0.0f);
            _vertices[0].Tu = 0;
            _vertices[0].Tv = 0;

            _vertices[1].Position = new Vector3(-5.0f * _aspectRatio, -5.0f, 0.0f);
            _vertices[1].Tu = 1;
            _vertices[1].Tv = 1;

            _vertices[2].Position = new Vector3(5.0f * _aspectRatio, -5.0f, 0.0f);
            _vertices[2].Tu = 0;
            _vertices[2].Tv = 1;

            _vertices[3].Position = new Vector3(-5.0f * _aspectRatio, -5.0f, 0.0f);
            _vertices[3].Tu = 1;
            _vertices[3].Tv = 1;

            _vertices[4].Position = new Vector3(5.0f * _aspectRatio, 5.0f, 0.0f);
            _vertices[4].Tu = 0;
            _vertices[4].Tv = 0;

            _vertices[5].Position = new Vector3(-5.0f * _aspectRatio, 5.0f, 0.0f);
            _vertices[5].Tu = 1;
            _vertices[5].Tv = 0;

            

            _texture = Direct3D.TextureLoader.FromFile(_device, "../../../texture1.bmp");
        }

        /// <summary>
        /// Release the resources of the texture.
        /// </summary>
        public void Shutdown()
        {
            if (_texture != null)
            {
                _texture.Dispose();
                _texture = null;
            }
        }
    }
}
