﻿using System.Drawing;  // needed for Size structure

namespace VDEngine.Game
{
    /// <summary>
    /// The user's display settings for the game.
    /// </summary>
    public class VDDisplaySettings
    {
        private bool _fullScreen;          // full-screen option
        private Size _windowSize;          // dimensions of the window
        private float _aspectRatio;        // aspect ratio (width / height)
        private float _nearClippingPlane;  // near clipping plane
        private float _farClippingPlane;   // far clipping plane



        #region Initialization (Constructor)

        /// <summary>
        /// Initializes the user's display settings.
        /// </summary>
        public VDDisplaySettings()
        {
            _fullScreen = false;
            _windowSize = new Size(1024, 768);
            _aspectRatio = (float) _windowSize.Width / (float) _windowSize.Height;
            _nearClippingPlane = 0.0f;
            _farClippingPlane = 500.0f;
        }

        #endregion



        #region Accessors & Modifiers

        /// <summary>
        /// Accessor and modifier for the full-screen or windowed option.
        /// </summary>
        public bool FullScreen
        {
            get
            {
                return _fullScreen;
            }
            set
            {
                _fullScreen = value;
            }
        }

        /// <summary>
        /// Accessor and modifier for the window's resolution.
        /// </summary>
        public Size WindowSize
        {
            get
            {
                return _windowSize;
            }
            set
            {
                _windowSize = value;
                _aspectRatio = (float) _windowSize.Width / (float) _windowSize.Height;
            }
        }

        /// <summary>
        /// Accessor for the window's aspect ratio.
        /// </summary>
        public float AspectRatio
        {
            get
            {
                return _aspectRatio;
            }
        }

        /// <summary>
        /// Accessor for the near clipping plane.
        /// </summary>
        public float NearClippingPlane
        {
            get
            {
                return _nearClippingPlane;
            }
        }

        /// <summary>
        /// Accessor for the far clipping plane.
        /// </summary>
        public float FarClippingPlane
        {
            get
            {
                return _farClippingPlane;
            }
        }

        #endregion
    }
}
